% LeaveOneOut() - Cross-validation technique that is a special case of the
%                 k-folds cross-validation. 'data' is a 2D array of feature
%                 vectors. 'labels' is a 1D array of class labels for the
%                 feature vectors. Each feature vector is classified from a
%                 LDA classifier trained from all other feature vectors.
%                 This process is repeated for each feature vector and the
%                 percentage of correctly classified feature vectors is
%                 returned as the OCR.
%
% Usage:
%   >> OCR = LeaveOneOut(data, labels)
%
% Inputs: 
%   data   - Nxl array of feature vectors where N is number of feature
%            vectors and l is number of features.
%   labels - N length array of class labels for feature vectors.
%
% Outputs:
%   OCR    - Overall classification rate of the LDA classifier with the
%            feature vector inputs of data. OCR = correctly classified
%            feature vectors / total number of feature vectors.
%
% Note: 
%
% Author: Andy Kerr, Autumn's Dawn NICE Lab, 2010
%
% See also: eegplugin_FST() classify()

function OCR = LeaveOneOut(data, labels)

global numobj
numobj=numobj+1;

correctlyClassified=0;
totalFV = size(data,1);
labelsize=size(labels);
if labelsize(1) == 1
    labels=labels';
end

%Leave One out runs through main loop once for each feature vector 
for i=1:size(data,1)
    %Test data consists of one complete FV
    testData=data(i,:);
    %Train data is all data excluding the single testing FV
    trainData=data;
    trainData(i,:)=[];
    %Train labels is used to determine the class of each FV in training set
    trainLabels=labels;
    trainLabels(i,:)=[];
    %The number of correctly classified test FVs
    try
        testClass = classify(testData,trainData,trainLabels);
    catch ME
        testClass=0;
    end
    if testClass == labels(i)
        correctlyClassified=correctlyClassified+1;
    end
end

OCR=correctlyClassified/totalFV;