% pop_coherence() - Generates feature vectors based on coherence in theta,
%                   delta, alpha, beta, and gamma ranges for different 
%                   channel pairs for each epoch. The user can select from
%                   all possible channel combinations or a list of default
%                   coherence pairs.
%
% Usage:
%   >> [ALLEEG com] = pop_coherence(ALLEEG, freq, cohtyp)
%
% Inputs: 
%   ALLEEG           - Input struct containing all loaded EEG sets.
%   freq(optional)   - Array of length 8 containing start and cuttoff 
%                      frequencies for delta, theta, alpha, beta, and gamma
%                      ranges respectivly.
%   cohtyp(optional) - integer input relating to coherence type
%                      1 - Far coherence
%                      2 - Anterior coherence
%                      3 - Local Posterior coherence
%                      4 - Posterior to Anterior coherence
%                      5 - Anterior to Posterior coherence
%                      6 - All possible channel pairs
%   montyp(optional) - integer input relating to the montage
%                      1 - 10-10
%                      2 - 10-20
%
% Outputs:
%   ALLEEG - Output struct containing all loaded EEG sets.
%   com    - Command tag.
%
% Note: 
%
% Author: Andy Kerr, Autumn's Dawn NICE Lab, 2010
%
% See also: eegplugin_FST() 

function [ALLEEG com] = pop_coherence(ALLEEG, freq, cohtyp, montyp)

global FV
FV.OCR=[];
com='';
FV.data=[];
%default badnwidths
%------------------
deltaRange=[1 4];
thetaRange=[4 7];
alphaRange=[8 12];
betaRange=[12 30];
gammaRange=[30 99];

%is input display needed?
%---------------------------
displayInput=0;
if nargin > 3
    if length(freq) ~= 8 
        displayInput = 1;
    end
    if cohtyp < 1 || cohtyp > 6
        displayInput = 1;
    end
    if montyp < 1 || montyp > 2
        displayInput = 1;
    end
end
if nargin < 3
    displayInput=1;
end
if displayInput
    cohtyps={'Far Coherence' 'Anterior Coherence' 'Local Posterior Coherence' ...
        'Posterior to Anterior Coherence' 'Anterior to Posterior Coherence' 'All possible pairs'};
    montageList={'10-10' '10-20'};
    %draw gui for entering bandwidths
    %--------------------------------
    uigeom = [5 5 5 5 5 4];
    uilist = { { 'style' 'text'      'string'    'Delta'} ...
        { 'style' 'edit'      'string'    num2str(deltaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(deltaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Theta'} ...
        { 'style' 'edit'      'string'    num2str(thetaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(thetaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Alpha'} ...
        { 'style' 'edit'      'string'    num2str(alphaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(alphaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Beta'} ...
        { 'style' 'edit'      'string'    num2str(betaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(betaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Gamma'} ...
        { 'style' 'edit'      'string'    num2str(gammaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(gammaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...        
        {'style'  'text'      'string'    'Coherence Type'} ...
        {'style'  'popupmenu' 'string'    cohtyps} ...
        {'style'  'text'      'string'    'Montage'} ...
        {'style'  'popupmenu' 'string'    montageList} ...
        };
    guititle = 'Coherence Feature Generation';
    result = inputgui( uigeom, uilist, 'pophelp(''pop_coherence'')', guititle, [], 'normal');
    if isempty(result)
        return; 
    end
    freq=result(1:10);
    cohtyp=result{11};
    montyp=result{12};
end

%update bandwidths to user defined from gui
%------------------------------------------
deltaRange = str2double(freq(1:2));
thetaRange = str2double(freq(3:4));
alphaRange = str2double(freq(5:6));
betaRange = str2double(freq(7:8));
gammaRange = str2double(freq(9:10));

%frequency band labels
%--------------------------------------
bands{1}='Delta';
bands{2}='Theta';
bands{3}='Alpha';
bands{4}='Beta';
bands{5}='Gamma';

%check that cancel was not pressed and all files have a class assigned
%-------------------------------------------------------
if ~isempty(result)
    for i=1:length(ALLEEG)
        try
            ALLEEG(i).class;
        catch ME
            disp('(pop_APM) error: Not all classes have labels'); return;
        end
    end
    
    %error checking from user input
    %--------------------------------
    fs=ALLEEG(1).srate;
    try
        if deltaRange(1) > deltaRange(2) 
            error('Delta bandwidth invalid.')
        end
        if thetaRange(1) > thetaRange(2) 
            error('Theta bandwidth invalid.')
        end
        if alphaRange(1) > alphaRange(2)
            error('Alpha bandwidth invalid.')
        end
        if betaRange(1) > betaRange(2) 
            error('Beta bandwidth invalid.')
        end
        if gammaRange(1) > gammaRange(2) 
            error('Gamma bandwidth invalid.')
        end
        if deltaRange(2) > fs/2 || thetaRange(2) > fs/2 || alphaRange(2) > fs/2 || betaRange(2) > fs/2 || gammaRange(2) > fs/2
            error(['Frequencies must be a number within the range of 1 to fs/2.  Current fs: ' num2str(fs)])
        end
    catch ME
        disp(['ERROR: ' ME.message]); return;
    end
    
    %create progress bar
    %------------------------------
    progress = waitbar(0,'Creating Data Set');

    %combine data from all files into one variable
    %--------------------------------
    minVec=zeros(length(ALLEEG));
    for i=1:length(ALLEEG)
        minVec(i)=size(ALLEEG(i).data,2);
    end
    minLength = min(minVec);
    data=ALLEEG(1).data(:,1:minLength,:);
    for i=2:length(ALLEEG)
        waitbar(i/(length(ALLEEG)*2),progress)
        data(:,1:minLength,end+1:end+size(ALLEEG(i).data,3))=ALLEEG(i).data(:,1:minLength,:);
    end
    
    %info on data
    %--------------------------------
    [numChannels epochLength numEpochs] = size(data);
    
    %find channels of interest
    %---------------------------
    labelInd=zeros(14,1);
    for i=1:size(ALLEEG(1).chanlocs,2)
        if strcmp(ALLEEG(1).chanlocs(i).labels,'O1')
            labelInd(1)=i;
        end
        if montyp == 1
            if strcmp(ALLEEG(1).chanlocs(i).labels,'FP1')
                labelInd(2)=i;
            end
        elseif montyp == 2
            if strcmp(ALLEEG(1).chanlocs(i).labels,'Fp1')
                labelInd(2)=i;
            end
        end
        if strcmp(ALLEEG(1).chanlocs(i).labels,'O2')
            labelInd(3)=i;
        end
        if montyp == 1
            if strcmp(ALLEEG(1).chanlocs(i).labels,'FP2')
                labelInd(4)=i;
            end
            if strcmp(ALLEEG(1).chanlocs(i).labels,'F7')
            labelInd(5)=i;
            end
            if strcmp(ALLEEG(1).chanlocs(i).labels,'F8')
                labelInd(6)=i;
            end
        elseif montyp == 2
            if strcmp(ALLEEG(1).chanlocs(i).labels,'Fp2')
                labelInd(4)=i;
            end
            if strcmp(ALLEEG(1).chanlocs(i).labels,'T1')
                labelInd(5)=i;
            end
            if strcmp(ALLEEG(1).chanlocs(i).labels,'T2')
                labelInd(6)=i;
            end
        end        
        if strcmp(ALLEEG(1).chanlocs(i).labels,'F3')
            labelInd(7)=i;
        end
        if strcmp(ALLEEG(1).chanlocs(i).labels,'F4')
            labelInd(8)=i;
        end
        if strcmp(ALLEEG(1).chanlocs(i).labels,'P3')
            labelInd(9)=i;
        end
        if strcmp(ALLEEG(1).chanlocs(i).labels,'P4')
            labelInd(10)=i;
        end
        if montyp == 1
            if strcmp(ALLEEG(1).chanlocs(i).labels,'P7')
                labelInd(11)=i;
            end
            if strcmp(ALLEEG(1).chanlocs(i).labels,'P8')
                labelInd(12)=i;
            end
        elseif montyp == 2
            if strcmp(ALLEEG(1).chanlocs(i).labels,'T5')
                labelInd(11)=i;
            end
            if strcmp(ALLEEG(1).chanlocs(i).labels,'T6')
                labelInd(12)=i;
            end
        end
        if strcmp(ALLEEG(1).chanlocs(i).labels,'C3')
            labelInd(13)=i;
        end
        if strcmp(ALLEEG(1).chanlocs(i).labels,'C4')
            labelInd(14)=i;
        end
    end
    
    %find Far coherence channel pairs if they exist in the ALLEEG(1).labels
    %-----------------------------------------------
    if cohtyp == 1
        selectedPairs=zeros(18,2);
        removeInd=1;
        if labelInd(1) ~= 0 && labelInd(2) ~=0
            selectedPairs(1,:)=[labelInd(1) labelInd(2)]; %O1-Fp1
        end
        if labelInd(3) ~= 0 && labelInd(4) ~=0
            selectedPairs(2,:)=[labelInd(3) labelInd(4)]; %O2-Fp2
        end
        if labelInd(1) ~= 0 && labelInd(5) ~=0
            selectedPairs(3,:)=[labelInd(1) labelInd(5)]; %O1-F7
        end
        if labelInd(3) ~= 0 && labelInd(6) ~=0
            selectedPairs(4,:)=[labelInd(3) labelInd(6)]; %O2-F8
        end
        if labelInd(5) ~= 0 && labelInd(7) ~=0
            selectedPairs(5,:)=[labelInd(1) labelInd(7)]; %O1-F3
        end
        if labelInd(3) ~= 0 && labelInd(8) ~=0
            selectedPairs(6,:)=[labelInd(3) labelInd(8)]; %O2-F4
        end
        if labelInd(9) ~= 0 && labelInd(2) ~=0
            selectedPairs(7,:)=[labelInd(9) labelInd(2)]; %P3-Fp1
        end
        if labelInd(10) ~= 0 && labelInd(4) ~=0
            selectedPairs(8,:)=[labelInd(10) labelInd(4)]; %P4-Fp2
        end
        if labelInd(9) ~= 0 && labelInd(5) ~=0
            selectedPairs(9,:)=[labelInd(9) labelInd(5)]; %P3-F7
        end
        if labelInd(10) ~= 0 && labelInd(6) ~=0
            selectedPairs(10,:)=[labelInd(10) labelInd(6)]; %P4-F8
        end
        if labelInd(9) ~= 0 && labelInd(7) ~=0
            selectedPairs(11,:)=[labelInd(9) labelInd(7)]; %P3-F3
        end
        if labelInd(10) ~= 0 && labelInd(8) ~=0
            selectedPairs(12,:)=[labelInd(10) labelInd(8)]; %P4-F4
        end
        if labelInd(11) ~= 0 && labelInd(2) ~=0
            selectedPairs(13,:)=[labelInd(11) labelInd(2)]; %P7-Fp1
        end
        if labelInd(12) ~= 0 && labelInd(4) ~=0
            selectedPairs(14,:)=[labelInd(12) labelInd(4)]; %P8-Fp2
        end
        if labelInd(11) ~= 0 && labelInd(5) ~=0
            selectedPairs(15,:)=[labelInd(11) labelInd(5)]; %P7-F7
        end
        if labelInd(12) ~= 0 && labelInd(6) ~=0
            selectedPairs(16,:)=[labelInd(12) labelInd(6)]; %P8-F8
        end
        if labelInd(11) ~= 0 && labelInd(7) ~=0
            selectedPairs(17,:)=[labelInd(11) labelInd(7)]; %P7-F3
        end
        if labelInd(12) ~= 0 && labelInd(8) ~=0
            selectedPairs(18,:)=[labelInd(12) labelInd(8)]; %P8-F4
        end
        indToRemove=[];
        for i=1:size(selectedPairs,1)
            if selectedPairs(i,1) == 0
                indToRemove(removeInd)=i;
                removeInd=removeInd+1;
            end
        end
        if ~isempty(indToRemove)
            selectedPairs(indToRemove,:)=[];
        end
        for i=1:size(selectedPairs,1)
            for j=1:5
                FV.Flabels{i+size(selectedPairs,1)*(j-1)} = [char(bands(j)) ' ' ALLEEG(1).chanlocs(selectedPairs(i,1)).labels '<->' ALLEEG(1).chanlocs(selectedPairs(i,2)).labels];
            end
        end
        FV.featureNumbers=1:size(selectedPairs,1)*5;
    end
    
    %find anterior coherence channel pairs if they exist in the ALLEEG(1).labels
    %-----------------------------------------------
    if cohtyp == 2
        selectedPairs=zeros(10,2);
        removeInd=1;
        if labelInd(2) ~= 0 && labelInd(5) ~=0
            selectedPairs(1,:)=[labelInd(2) labelInd(5)]; %Fp1-F7
        end
        if labelInd(4) ~= 0 && labelInd(6) ~=0
            selectedPairs(2,:)=[labelInd(4) labelInd(6)]; %Fp2-F8
        end
        if labelInd(2) ~= 0 && labelInd(7) ~=0
            selectedPairs(3,:)=[labelInd(2) labelInd(7)]; %Fp1-F3
        end
        if labelInd(4) ~= 0 && labelInd(8) ~=0
            selectedPairs(4,:)=[labelInd(4) labelInd(8)]; %Fp2-F4
        end
        if labelInd(2) ~= 0 && labelInd(13) ~=0
            selectedPairs(5,:)=[labelInd(2) labelInd(13)]; %Fp1-C3
        end
        if labelInd(4) ~= 0 && labelInd(14) ~=0
            selectedPairs(6,:)=[labelInd(4) labelInd(14)]; %Fp2-C4
        end
        if labelInd(5) ~= 0 && labelInd(13) ~=0
            selectedPairs(7,:)=[labelInd(5) labelInd(13)]; %F7-C3
        end
        if labelInd(6) ~= 0 && labelInd(14) ~=0
            selectedPairs(8,:)=[labelInd(6) labelInd(14)]; %F8-C4
        end
        if labelInd(7) ~= 0 && labelInd(13) ~=0
            selectedPairs(9,:)=[labelInd(7) labelInd(13)]; %F3-C3
        end
        if labelInd(8) ~= 0 && labelInd(14) ~=0
            selectedPairs(10,:)=[labelInd(8) labelInd(14)]; %F4-C4
        end

        indToRemove=[];
        for i=1:size(selectedPairs,1)
            if selectedPairs(i,1) == 0
                indToRemove(removeInd)=i;
                removeInd=removeInd+1;
            end
        end
        if ~isempty(indToRemove)
            selectedPairs(indToRemove,:)=[];
        end
        for i=1:size(selectedPairs,1)
            for j=1:5
                FV.Flabels{i+size(selectedPairs,1)*(j-1)} = [char(bands(j)) ' ' ALLEEG(1).chanlocs(selectedPairs(i,1)).labels '<->' ALLEEG(1).chanlocs(selectedPairs(i,2)).labels];
            end
        end
        FV.featureNumbers=1:size(selectedPairs,1)*5;
    end

    %find local posterior coherence channel pairs if they exist in the ALLEEG(1).labels
    %-----------------------------------------------
    if cohtyp == 3
        selectedPairs=zeros(10,2);
        removeInd=1;
        if labelInd(1) ~= 0 && labelInd(9) ~=0
            selectedPairs(1,:)=[labelInd(1) labelInd(9)]; %O1-P3
        end
        if labelInd(3) ~= 0 && labelInd(10) ~=0
            selectedPairs(2,:)=[labelInd(3) labelInd(10)]; %O2-P4
        end
        if labelInd(1) ~= 0 && labelInd(11) ~=0
            selectedPairs(3,:)=[labelInd(1) labelInd(11)]; %O1-P7
        end
        if labelInd(3) ~= 0 && labelInd(12) ~=0
            selectedPairs(4,:)=[labelInd(3) labelInd(12)]; %O2-P8
        end
        if labelInd(1) ~= 0 && labelInd(13) ~=0
            selectedPairs(5,:)=[labelInd(1) labelInd(13)]; %O1-C3
        end
        if labelInd(3) ~= 0 && labelInd(14) ~=0
            selectedPairs(6,:)=[labelInd(3) labelInd(14)]; %O2-C4
        end
        if labelInd(9) ~= 0 && labelInd(13) ~=0
            selectedPairs(7,:)=[labelInd(9) labelInd(13)]; %P3-C3
        end
        if labelInd(10) ~= 0 && labelInd(14) ~=0
            selectedPairs(8,:)=[labelInd(10) labelInd(14)]; %P4-C4
        end
        if labelInd(11) ~= 0 && labelInd(13) ~=0
            selectedPairs(9,:)=[labelInd(11) labelInd(13)]; %P7-C3
        end
        if labelInd(12) ~= 0 && labelInd(14) ~=0
            selectedPairs(10,:)=[labelInd(12) labelInd(14)]; %P8-C4
        end

        indToRemove=[];
        for i=1:size(selectedPairs,1)
            if selectedPairs(i,1) == 0
                indToRemove(removeInd)=i;
                removeInd=removeInd+1;
            end
        end
        if ~isempty(indToRemove)
            selectedPairs(indToRemove,:)=[];
        end
        for i=1:size(selectedPairs,1)
            for j=1:5
                FV.Flabels{i+size(selectedPairs,1)*(j-1)} = [char(bands(j)) ' ' ALLEEG(1).chanlocs(selectedPairs(i,1)).labels '<->' ALLEEG(1).chanlocs(selectedPairs(i,2)).labels];
            end
        end
        FV.featureNumbers=1:size(selectedPairs,1)*5;
    end

    %find posterior to anterior coherence channel pairs if they exist in the ALLEEG(1).labels
    %-----------------------------------------------
    if cohtyp == 4
        selectedPairs=zeros(8,2);
        removeInd=1;
        if labelInd(1) ~= 0 && labelInd(2) ~=0
            selectedPairs(1,:)=[labelInd(1) labelInd(2)]; %O1-Fp1
        end
        if labelInd(3) ~= 0 && labelInd(4) ~=0
            selectedPairs(2,:)=[labelInd(3) labelInd(4)]; %O2-Fp2
        end
        if labelInd(1) ~= 0 && labelInd(7) ~=0
            selectedPairs(3,:)=[labelInd(1) labelInd(7)]; %O1-F3
        end
        if labelInd(3) ~= 0 && labelInd(8) ~=0
            selectedPairs(4,:)=[labelInd(3) labelInd(8)]; %O2-F4
        end
        if labelInd(1) ~= 0 && labelInd(13) ~=0
            selectedPairs(5,:)=[labelInd(1) labelInd(13)]; %O1-C3
        end
        if labelInd(3) ~= 0 && labelInd(14) ~=0
            selectedPairs(6,:)=[labelInd(3) labelInd(14)]; %O2-C4
        end
        if labelInd(1) ~= 0 && labelInd(9) ~=0
            selectedPairs(7,:)=[labelInd(1) labelInd(9)]; %O1-P3
        end
        if labelInd(3) ~= 0 && labelInd(10) ~=0
            selectedPairs(8,:)=[labelInd(3) labelInd(10)]; %O1-P4
        end

        indToRemove=[];
        for i=1:size(selectedPairs,1)
            if selectedPairs(i,1) == 0
                indToRemove(removeInd)=i;
                removeInd=removeInd+1;
            end
        end
        if ~isempty(indToRemove)
            selectedPairs(indToRemove,:)=[];
        end
        for i=1:size(selectedPairs,1)
            for j=1:5
                FV.Flabels{i+size(selectedPairs,1)*(j-1)} = [char(bands(j)) ' ' ALLEEG(1).chanlocs(selectedPairs(i,1)).labels '<->' ALLEEG(1).chanlocs(selectedPairs(i,2)).labels];
            end
        end
        FV.featureNumbers=1:size(selectedPairs,1)*5;
    end
    
    %find anterior to posterior coherence channel pairs if they exist in the ALLEEG(1).labels
    %-----------------------------------------------
    if cohtyp == 5
        selectedPairs=zeros(8,2);
        removeInd=1;
        if labelInd(1) ~= 0 && labelInd(2) ~=0
            selectedPairs(1,:)=[labelInd(2) labelInd(1)]; %Fp1-O1
        end
        if labelInd(3) ~= 0 && labelInd(4) ~=0
            selectedPairs(2,:)=[labelInd(4) labelInd(3)]; %Fp2-O2
        end
        if labelInd(1) ~= 0 && labelInd(7) ~=0
            selectedPairs(3,:)=[labelInd(2) labelInd(9)]; %Fp1-P3
        end
        if labelInd(3) ~= 0 && labelInd(8) ~=0
            selectedPairs(4,:)=[labelInd(4) labelInd(10)]; %Fp2-P4
        end
        if labelInd(1) ~= 0 && labelInd(13) ~=0
            selectedPairs(5,:)=[labelInd(2) labelInd(13)]; %Fp1-C3
        end
        if labelInd(3) ~= 0 && labelInd(14) ~=0
            selectedPairs(6,:)=[labelInd(4) labelInd(14)]; %Fp2-C4
        end
        if labelInd(1) ~= 0 && labelInd(9) ~=0
            selectedPairs(7,:)=[labelInd(2) labelInd(7)]; %Fp1-F3
        end
        if labelInd(3) ~= 0 && labelInd(10) ~=0
            selectedPairs(8,:)=[labelInd(4) labelInd(8)]; %Fp2-F4
        end

        indToRemove=[];
        for i=1:size(selectedPairs,1)
            if selectedPairs(i,1) == 0
                indToRemove(removeInd)=i;
                removeInd=removeInd+1;
            end
        end
        if ~isempty(indToRemove)
            selectedPairs(indToRemove,:)=[];
        end
        for i=1:size(selectedPairs,1)
            for j=1:5
                FV.Flabels{i+size(selectedPairs,1)*(j-1)} = [char(bands(j)) ' ' ALLEEG(1).chanlocs(selectedPairs(i,1)).labels '<->' ALLEEG(1).chanlocs(selectedPairs(i,2)).labels];
            end
        end
        FV.featureNumbers=1:size(selectedPairs,1)*5;
    end
    if cohtyp==6
        k=1;
        selectedPairs=zeros(numChannels*2,2);
        for i=1:numChannels-1
            for j=i+1:numChannels
                selectedPairs(k,:)=[i j];
                k=k+1;
            end
        end
        for i=1:size(selectedPairs,1)
            for j=1:5
                FV.Flabels{i+size(selectedPairs,1)*(j-1)} = [char(bands(j)) ' ' ALLEEG(1).chanlocs(selectedPairs(i,1)).labels '<->' ALLEEG(1).chanlocs(selectedPairs(i,2)).labels];
            end
        end
        FV.featureNumbers=1:size(selectedPairs,1)*5;
    end
    
    %create progress bar
    %------------------------------
    progress = waitbar(1/2,progress,'Generating Features');
    tic;
    %generate coherence features
    %-----------------------------------------
    for i=1:size(selectedPairs,1)
        for k=1:numEpochs
            waitbar(1/2+k*i/(size(selectedPairs,1)*numEpochs)/2,progress)

            [C f] = mscohere(data(selectedPairs(i,1),:,k),data(selectedPairs(i,2),:,k),[],[],[],fs);
            [mindif, deltaStart] = min(abs(f - deltaRange(1)));
            [mindif, deltaCutoff] = min(abs(f - deltaRange(2)));

            [mindif, thetaStart] = min(abs(f - thetaRange(1)));
            [mindif, thetaCutoff] = min(abs(f - thetaRange(2)));

            [mindif, alphaStart] = min(abs(f - alphaRange(1)));
            [mindif, alphaCutoff] = min(abs(f - alphaRange(2)));

            [mindif, betaStart] = min(abs(f - betaRange(1)));
            [mindif, betaCutoff] = min(abs(f - betaRange(2)));
            
            [mindif, gammaStart] = min(abs(f - gammaRange(1)));
            [mindif, gammaCutoff] = min(abs(f - gammaRange(2)));

            FV.data(k,i)=sum(C(deltaStart:deltaCutoff))/(deltaCutoff-deltaStart);

            FV.data(k,i+size(selectedPairs,1))=sum(C(thetaStart:thetaCutoff))/(thetaCutoff-thetaStart);

            FV.data(k,i+size(selectedPairs,1)*2)=sum(C(alphaStart:alphaCutoff))/(alphaCutoff-alphaStart);

            FV.data(k,i+size(selectedPairs,1)*3)=sum(C(betaStart:betaCutoff))/(betaCutoff-betaStart);
            
            FV.data(k,i+size(selectedPairs,1)*4)=sum(C(gammaStart:gammaCutoff))/(gammaCutoff-gammaStart);
        end
    end
    toc;
    close(progress)
end