% pop_phaseSynchrony() - Generates feature vectors based on phase synchrony 
%              in theta, delta, alpha, beta, and gamma ranges for all 
%              possible channel pairs for each epoch.
%
% Usage:
%   >> [ALLEEG com] = pop_phaseSynchrony(ALLEEG, freq)
%
% Inputs: 
%   ALLEEG         - Input struct containing all loaded EEG sets.
%   freq(optional) - Array of length 8 containing start and cuttoff 
%                    frequencies for delta, theta, alpha, beta ranges
%                    respectivly.
%
% Outputs:
%   ALLEEG - Output struct containing all loaded EEG sets.
%   com    - Command tag.
%
% Note: 
%
% Author: Andy Kerr, Autumn's Dawn NICE Lab, 2011
%
% See also: eegplugin_FST() 

function [ALLEEG, com] = pop_phaseSynchrony(ALLEEG, freq)

global FV

com='';
FV.data=[];
%default badnwidths
%------------------
deltaRange=[0.5 4];
thetaRange=[4 7];
alphaRange=[8 12];
betaRange=[12 30];
gammaRange=[30 99];

%is input display needed?
%---------------------------
displayInput=0;
if nargin > 1
    if length(freq) ~= 8 
        displayInput = 1;
    end
end
if nargin == 1
    displayInput = 1;
end
if displayInput
    %draw gui for entering bandwidths
    %--------------------------------
    uigeom = [5 5 5 5 5];
    uilist = { { 'style' 'text'      'string'    'Delta'} ...
        { 'style' 'edit'      'string'    num2str(deltaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(deltaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Theta'} ...
        { 'style' 'edit'      'string'    num2str(thetaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(thetaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Alpha'} ...
        { 'style' 'edit'      'string'    num2str(alphaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(alphaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Beta'} ...
        { 'style' 'edit'      'string'    num2str(betaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(betaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        { 'style' 'text'      'string'    'Gamma'} ...
        { 'style' 'edit'      'string'    num2str(gammaRange(1))} ...
        {'style'  'text'      'string'    'to'} ...
        { 'style' 'edit'      'string'    num2str(gammaRange(2))} ...
        {'style'  'text'      'string'    'Hz'} ...
        };
    guititle = 'Phase Synchrony Feature Generation';
    freq = inputgui( uigeom, uilist, 'pophelp(''pop_phaseSynchrony'')', guititle, [], 'normal');  
end
if isempty(freq)
    return;
end

%update bandwidths to user defined from gui
%------------------------------------------
deltaRange = str2double(freq(1:2));
thetaRange = str2double(freq(3:4));
alphaRange = str2double(freq(5:6));
betaRange = str2double(freq(7:8));
gammaRange = str2double(freq(9:10));

%check that all files have a class assigned
%-------------------------------------------------------
for i=1:length(ALLEEG)
    try
        ALLEEG(i).class;
    catch ME
        disp('(pop_APM) error: Not all classes have labels'); return;
    end
end

%filter information
%--------------------------------
fs=ALLEEG(1).srate;  %sampling rate
order = 128;  %filter order

%create progress bar
%------------------------------
progress = waitbar(0,'Creating Data Set');

%combine data from all files into one variable
%--------------------------------
minVec=zeros(length(ALLEEG));
for i=1:length(ALLEEG)
    minVec(i)=size(ALLEEG(i).data,2);
end
minLength = min(minVec);
data=ALLEEG(1).data(:,1:minLength,:);
for i=2:length(ALLEEG)
    waitbar(i/(length(ALLEEG)*3),progress)
    data(:,1:minLength,end+1:end+size(ALLEEG(i).data,3))=ALLEEG(i).data(:,1:minLength,:);
end

%info on data
%--------------------------------
[numChannels epochLength numEpochs] = size(data);

%error checking from user input
%--------------------------------
try
    if deltaRange(1) > deltaRange(2) 
        error('Delta bandwidth invalid.')
    end
    if thetaRange(1) > thetaRange(2) 
        error('Theta bandwidth invalid.')
    end
    if alphaRange(1) > alphaRange(2)
        error('Alpha bandwidth invalid.')
    end
    if betaRange(1) > betaRange(2) 
        error('Beta bandwidth invalid.')
    end
    if gammaRange(1) > gammaRange(2) 
        error('Gamma bandwidth invalid.')
    end
    if deltaRange(2) > fs/2 || thetaRange(2) > fs/2 || alphaRange(2) > fs/2 || betaRange(2) > fs/2 || gammaRange(2) > fs/2
        error(['Frequencies must be a number within the range of 1 to fs/2.  Current fs: ' num2str(fs)])
    end
catch ME
    disp(['ERROR: ' ME.message]); return;
end

waitbar(1/3,progress,'Filtering Data')
%Filter data for Delta range
%-----------------------------
wn_Delta = deltaRange/(fs/2);
b = fir1(order,wn_Delta);
data_Delta = filter(b,1,data);
waitbar(1/3+1/3/5,progress);

%Filter data for Theta range
%------------------------------
wn_Theta = thetaRange/(fs/2);
b = fir1(order,wn_Theta);
data_Theta = filter(b,1,data);
waitbar(1/3+2/3/5,progress);

%Filter data for Alpha range
%------------------------------
wn_Alpha = alphaRange/(fs/2);
b = fir1(order,wn_Alpha);
data_Alpha = filter(b,1,data);
waitbar(1/3+3/3/5,progress);

%Filter data for Beta range
%------------------------------
wn_Beta = betaRange/(fs/2);
b = fir1(order,wn_Beta);
data_Beta = filter(b,1,data);
waitbar(1/3+4/3/5,progress);

%Filter data for Gamma range
%------------------------------
wn_Gamma = gammaRange/(fs/2);
b = fir1(order,wn_Gamma);
data_Gamma = filter(b,1,data);
waitbar(1/3+5/3/5,progress);

feature_labels={}; 

%create feature vectors
%------------------------------
waitbar(2/3,progress,'Generating Features');
tic;
FV.OCR=[];
numChannels_pairs=(numChannels-1)*(numChannels)/2;
FV.featureNumbers=1:numChannels_pairs*5;
FV.data=zeros(numEpochs,numChannels_pairs*5);
count=1;
%calculate correlation for all channel pairs
%-------------------------------
for i=1:numChannels-1
    for k=i+1:numChannels
        %generate feature labels
        %-----------------------
        feature_labels{count}=['Delta ' char(ALLEEG(1).chanlocs(i).labels) '<->' char(ALLEEG(1).chanlocs(k).labels)];
        feature_labels{count+numChannels_pairs}=['Theta ' char(ALLEEG(1).chanlocs(i).labels) '<->' char(ALLEEG(1).chanlocs(k).labels)];
        feature_labels{count+numChannels_pairs*2}=['Alpha ' char(ALLEEG(1).chanlocs(i).labels) '<->' char(ALLEEG(1).chanlocs(k).labels)];
        feature_labels{count+numChannels_pairs*3}=['Beta ' char(ALLEEG(1).chanlocs(i).labels) '<->' char(ALLEEG(1).chanlocs(k).labels)];
        feature_labels{count+numChannels_pairs*4}=['Gamma ' char(ALLEEG(1).chanlocs(i).labels) '<->' char(ALLEEG(1).chanlocs(k).labels)];
        for j=1:numEpochs
            waitbar(2/3+(j*i/(numChannels*numEpochs))/3,progress)
            %Delta
            %--------------------------------------------
            inst_phase_X = phase(hilbert(data_Delta(i,1:epochLength,j)));
            inst_phase_Y = phase(hilbert(data_Delta(k,1:epochLength,j)));
            phase_sync = abs(sum(exp(1j*(inst_phase_X-inst_phase_Y)))/length(inst_phase_X));
            FV.data(j,count) = phase_sync;              

            %Theta
            %--------------------------------------------
            inst_phase_X = phase(hilbert(data_Theta(i,1:epochLength,j)));
            inst_phase_Y = phase(hilbert(data_Theta(k,1:epochLength,j)));
            phase_sync = abs(sum(exp(1j*(inst_phase_X-inst_phase_Y)))/length(inst_phase_X));
            FV.data(j,count+numChannels_pairs) = phase_sync;

            %Alpha
            %--------------------------------------------
            inst_phase_X = phase(hilbert(data_Alpha(i,1:epochLength,j)));
            inst_phase_Y = phase(hilbert(data_Alpha(k,1:epochLength,j)));
            phase_sync = abs(sum(exp(1j*(inst_phase_X-inst_phase_Y)))/length(inst_phase_X));
            FV.data(j,count+numChannels_pairs*2) = phase_sync;

            %Beta
            %-------------------------------------------
            inst_phase_X = phase(hilbert(data_Beta(i,1:epochLength,j)));
            inst_phase_Y = phase(hilbert(data_Beta(k,1:epochLength,j)));
            phase_sync = abs(sum(exp(1j*(inst_phase_X-inst_phase_Y)))/length(inst_phase_X));
            FV.data(j,count+numChannels_pairs*3) = phase_sync;

            %Gamma
            %-------------------------------------------
            inst_phase_X = phase(hilbert(data_Gamma(i,1:epochLength,j)));
            inst_phase_Y = phase(hilbert(data_Gamma(k,1:epochLength,j)));
            phase_sync = abs(sum(exp(1j*(inst_phase_X-inst_phase_Y)))/length(inst_phase_X));
            FV.data(j,count+numChannels_pairs*4) = phase_sync;  
        end
        count=count+1;
    end
end
FV.Flabels=feature_labels;
toc;
close(progress)
%     com = sprintf( 'test');
return;

