% pop_stepwiseRegression() - Sequential feature reduction algorithm that 
%              takes a feature set and reduces the number of features to be
%              a quasi-optimal subset for classification with a length 
%              specified by the user.  
%
%              The following algorithms can be used:
%              SFS  - Sequential Forward Selection
%              SFFS - Sequential Forward Floating Selection
%              IFFS - Improved Forward Floating Selection
%
% Usage:
%   >> [ALLEEG com] = pop_SFFS(ALLEEG, maxFeatures, SFSmode)
%
% Inputs: 
%   ALLEEG      - Input struct containing all loaded EEG sets.
%   maxFeatures - Maximum feature set size to be returned
%   SFSmode     - String determining the algorithm to use
%
% Outputs:
%   ALLEEG - Output struct containing all loaded EEG sets.
%   com    - Command tag.
%
% Note: 
%
% Author: Andy Kerr, Autumn's Dawn NICE Lab, 2010
%
% See also: eegplugin_FST() stepwiseRegression() SFS() SBS()

function [ALLEEG com] = pop_stepwiseRegression(ALLEEG, maxFeatures, SFSmode)

com='';

global numobj
numobj=0;

global FV

%make sure features are generated
%----------------------------------
if isempty(FV.data)
    disp('(pop_SFFS) error: No Feature Vectors'); return;
end

%generate labels for each class
%---------------------------------
classLabels=[];

for i=1:length(ALLEEG)
    classLabels(end+1:end+size(ALLEEG(i).data,3))=ALLEEG(i).class;
end

if nargin < 2
    %draw GUI for feature reduction
    %-------------------------------
    uigeom=[2 2 2];
    maxFeatures=5;
    SFSmodes={'SFS' 'SFFS' 'IFFS'};
    valModes={'Basic' 'K-folds' 'Leave One Out'};

    uilist = { { 'style' 'text'      'string'    'Mode: '} ...
        { 'style' 'popupmenu'      'string'    SFSmodes} ...
        { 'style' 'text'      'string'    'Max Features: '} ...
        { 'style' 'edit'      'string'    maxFeatures} ...
        { 'style' 'text'      'string'    'Validation: '} ...
        { 'style' 'popupmenu'      'string'    valModes} ...
        };

    guititle = 'Sequential Feature Selection';
    result = inputgui( uigeom, uilist, 'pophelp(''pop_SFFS'')', guititle, [], 'normal');
end

%check valid input
%---------------------------------
if isempty(result)
    return;
end
part={};
if isnan(str2double(result(2))) && ~isreal(str2double(result(2)))
    disp('(pop_SFFS) error: Invalid Input'); return;
end

if result{3}==2
    uilist={};
    %draw gui for entering number of classes
    %--------------------------------
    uigeom = 2;
    numFolds=10;

    uilist{end+1} = { 'style' 'text'      'string'    'Number of Folds: '};
    uilist{end+1} = { 'style' 'edit'      'string'    num2str(numFolds)};

    guititle = 'k-folds';
    numClassesStr = inputgui( uigeom, uilist, 'pophelp(''pop_APM'')', guititle, [], 'normal');
    if isempty(numClassesStr)
        return;
    end
    numFolds=str2double(numClassesStr{1});

    totalFV = size(FV.data,1);
    partSize=floor(totalFV/numFolds);
    leftover=mod(totalFV,numFolds);
    allind=randperm(totalFV);
    part={};
    for i=1:numFolds
        if i <= leftover
            part{i}=allind(1+(i-1)*(partSize+1):i*(partSize+1));
        else
            part{i}=allind((1+(i-1)*partSize)+leftover:i*partSize+leftover);
        end
    end
end


FV.featureNumbers=[];

% tic;
[FV.data FV.featureNumbers FV.OCR]=stepwiseRegression(FV.data,classLabels,SFSmodes{result{1}},str2double(result{2}),result{3},part);
% toc;

% numobj

uigeom=[1 1];

uilist = { { 'style' 'text'      'string'    ['OCR: ' num2str(FV.OCR*100) '%']} ...
    { 'style' 'text'      'string'    ['Mode Used: ' SFSmodes{result{1}}]} ...
    };

guititle = 'Selection Results';
inputgui( uigeom, uilist, 'pophelp(''pop_stepwiseRegression'')', guititle, [], 'normal');


