% pop_wavlet() - Generates feature vectors based on CWT
%
% Usage:
%   >> [ALLEEG com] = pop_wavelet(ALLEEG, scales)
%
% Inputs: 
%   ALLEEG         - Input struct containing all loaded EEG sets.
%   scales(optional) - wavelet scales
%
% Outputs:
%   ALLEEG - Output struct containing all loaded EEG sets.
%   com    - Command tag.
%
% Note: 
%
% Author: Andy Kerr, Autumn's Dawn NICE Lab, 2011
%
% See also: eegplugin_SFT() 

function [ALLEEG, com] = pop_wavelet(ALLEEG, scales)

global FV

com='';
FV.data=[];
%default badnwidths
%------------------
scales=[2 4 8 16 32 64 128];


%is input display needed?
%---------------------------
displayInput=0;
if nargin > 1
    if length(freq) ~= 8 
        displayInput = 1;
    end
end
if nargin == 1
    displayInput = 1;
end
if displayInput
    %draw gui for entering bandwidths
    %--------------------------------
    uigeom = [2];
    uilist = { { 'style' 'text'      'string'    'Scales'} ...
        { 'style' 'edit'      'string'    num2str(scales)} ...
        };
    guititle = 'Wavelet Feature Generation';
    newscales = inputgui( uigeom, uilist, 'pophelp(''pop_wavelet'')', guititle, [], 'normal');  
end
if isempty(newscales)
    return;
end

%update bandwidths to user defined from gui
%------------------------------------------
scales = str2num(newscales{1});

%check that all files have a class assigned
%-------------------------------------------------------
for i=1:length(ALLEEG)
    try
        ALLEEG(i).class;
    catch ME
        disp('(pop_APM) error: Not all classes have labels'); return;
    end
end

%create progress bar
%------------------------------
progress = waitbar(0,'Creating Data Set');

%combine data from all files into one variable
%--------------------------------
minVec=zeros(length(ALLEEG));
for i=1:length(ALLEEG)
    minVec(i)=size(ALLEEG(i).data,2);
end
minLength = min(minVec);
data=ALLEEG(1).data(:,1:minLength,:);
for i=2:length(ALLEEG)
    waitbar(i/(length(ALLEEG)*3),progress)
    data(:,1:minLength,end+1:end+size(ALLEEG(i).data,3))=ALLEEG(i).data(:,1:minLength,:);
end

%info on data
%--------------------------------
[numChannels epochLength numEpochs] = size(data);

feature_labels={};   
speriod=1/ALLEEG(1).srate;

%create feature vectors
%------------------------------
waitbar(2/3,progress,'Generating Features');
numdecomps=size(scales,2);
FV.data=zeros(numEpochs,numChannels*numdecomps);
FV.featureNumbers=1:numChannels*numdecomps;
FV.OCR=[];
for i=1:numChannels
    for j=1:numEpochs
        coefs = CWT(data(i,:,j),scales,'db5'); 
        F = scal2frq(scales,'db5',speriod)
        for k=1:numdecomps
            feature_labels{k+numdecomps*(i-1)}=['Level ' num2str(scales(k)) ' Channel ' ALLEEG(1).chanlocs(i).labels];
            avg=sum(coefs(k))/size(coefs(k),1);
            FV.data(j,k+numdecomps*(i-1)) = avg;
        end   
    end
end
FV.Flabels=feature_labels;
close(progress)
%     com = sprintf( 'test');
return;