% stepwiseRegression() - Optimally reduce the subset based on stepwise
%                        regression
%
% Usage:
%   >> [FV_Reduced pickedFeatures] = stepwiseRegression(FV, labels, mode, maxFeatures,val,part)
%
% Inputs: 
%   FV     - Nxl array of feature vectors where N is number of feature
%            vectors and l is number of features.
%   labels - N length array class labels for feature vectors, 2 class only, 
%            1's for class 1 and 2's for class 2
%   mode   - 'SFS'  - Sequential Forward Selection
%            'SFFS' - Sequential Forward Floating Selection
%            'IFFS' - Improved Forward Floating Selection
%   maxFeatures - maximum features to select
%   val    - integer representing validation type
%             1 - test-set
%             2 - kfolds
%             3 - leave-one-out
%   part   - randomly generated indices from 1 to N
%
% Outputs:
%   FV_Reduced     - Nxl array of feature vectors where N is the number of
%                    feature vectores and l is hte number of features 
%                    picked by SFFS.
%   pickedFeatures - Array of picked features.
%
% Note: 
%
% Author: Andy Kerr, Autumn's Dawn NICE Lab, 2010
%
% See also: eegplugin_SFT() pop_stepwiseRegression()

function [FV_Reduced pickedFeatures maxOCR] = stepwiseRegression(FV, labels, mode, maxFeatures,val,part)

[numQuestions numFeatures] = size(FV);
%A list of the features to keep track of which need to be checked
featureIndex=1:numFeatures;
%User defined input of max features
%Two checks are preformed on this number
%  The first checks if the number of max features is greater than or equal
%  to the number of total features, if so the max features is set to the
%  total number of features minus one
%  The Second check makes sure that there are enough questions for the
%  features given.  If there are not enough FVs then there will be an error
%  with the LDA classifier.

delta=5;
if strcmp(mode,'SFS')
    delta=0;
end
if maxFeatures+delta >= numFeatures
    maxFeatures=numFeatures-1;
end

progress = waitbar(0,'Feature Reduction Progress');
k=0;
OCR=[];

global numobj

features=zeros(maxFeatures+delta);
tic;
%should exit when SBS stops kicking in and we get to the end of features
while k < maxFeatures+delta 
    %SFS
    [nextFeature OCR(k+1)] = SFS(features, featureIndex, FV, labels,k,val,part);
    if features(1,1) ~= 0
        features(k+1,1:k+1)=[features(k,1:k) nextFeature];
    else
        features(k+1,1)=nextFeature;
    end
    
    k=k+1;
    waitbar(k/(maxFeatures+delta))
    if k >= maxFeatures+delta
        break;
    end
    if strcmp(mode,'SFFS') || strcmp(mode,'IFFS')
        %SBS
        [SBSfeatures SBSOCR] = SBS(features, FV, k, labels,val,part);
        %Is SBS better than OCR(k-1)?
        if k>1
            while SBSOCR > OCR(k-1)
                features(k-1,1:k-1)=SBSfeatures;
                OCR(k-1)=SBSOCR;
                k=k-1;
                [SBSfeatures SBSOCR] = SBS(features, FV, k, labels,val,part);
            end
        end
    end
    if strcmp(mode,'IFFS')
        if k>1
            impFeature=zeros(k,1);
            impOCR=zeros(k,1);
            for i=1:k
                tempFeatures=features;
                tempFeatures(k-1,1:k-1)=tempFeatures(k,[1:i-1 i+1:k]);
                [impFeature(i) impOCR(i)] = SFS(tempFeatures, featureIndex, FV, labels,k-1,val,part);
            end
            if max(impOCR) > OCR(k)
                tempFeatures1=features(k,1:k);
                [impOCRsingle impInd] = max(impOCR);
                tempFeatures1(impInd)=[];
                features(k,1:k) = [tempFeatures1 impFeature(impInd)];
            end
        end
    end
%     OCR
%     numobj
%     toc;
end
[maxOCR maxOCRIndex] =max(OCR(1:maxFeatures));
FV_Reduced=FV(:,features(maxOCRIndex,1:maxOCRIndex));
pickedFeatures = features(maxOCRIndex,1:maxOCRIndex);
close(progress)
